@@ -1,8 +1,16 @@
 package Google::RestApi::Auth;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.4';
+
+use 5.010_000;
+
+use autodie;
+no autovivification;
+
+do 'Google/RestApi/logger_init.pl';
 
 sub params {{}}
 sub headers {[]};
