@@ -1,33 +1,41 @@
 package Google::RestApi::Auth::ServiceAccount;
 
-our $VERSION = '0.9.1';
+use strict;
+use warnings;
 
-use Google::RestApi::Setup;
+our $VERSION = '0.4';
 
-use WWW::Google::Cloud::Auth::ServiceAccount ();
+use 5.010_000;
+
+use autodie;
+use Type::Params qw(compile_named);
+use Types::Standard qw(Str ArrayRef);
+use WWW::Google::Cloud::Auth::ServiceAccount;
+use YAML::Any qw(Dump);
+
+no autovivification;
+
+use Google::RestApi::Utils qw(config_file resolve_config_file);
 
 use parent 'Google::RestApi::Auth';
 
+do 'Google/RestApi/logger_init.pl';
+
 sub new {
   my $class = shift;
 
-  my %p = @_;
-  # order is important, resolve the overall config file first.
-  resolve_config_file_path(\%p, 'config_file');
-  resolve_config_file_path(\%p, 'account_file');
-  
-  my $self = config_file(%p);
+  my $self = config_file(@_);
   state $check = compile_named(
-    config_dir   => ReadableDir, { optional => 1 },
-    config_file  => ReadableFile, { optional => 1 },
-    account_file => ReadableFile,
-    scope        => ArrayRef[Str],
+    config_file        => Str, { optional => 1 },
+    parent_config_file => Str, { optional => 1 },  # only used internally
+    account_file       => Str,
+    scope              => ArrayRef[Str],
   );
   $self = $check->(%$self);
   $self = bless $self, $class;
 
   my $auth = WWW::Google::Cloud::Auth::ServiceAccount->new(
-    credentials_path => $self->{account_file},
+    credentials_path => $self->account_file(),
     # undocumented feature of WWW::Google::Cloud::Auth::ServiceAccount
     scope            => join(' ', @{ $self->{scope} }),
   );
@@ -46,10 +54,17 @@ sub headers {
 sub access_token {
   my $self = shift;
   $self->{access_token} = $self->{auth}->get_token()
-    or LOGDIE "Service Account Auth failed";
+    or die "Service Account Auth failed";
   return $self->{access_token};
 }
 
+sub account_file {
+  my $self = shift;
+  $self->{_account_file} = resolve_config_file('account_file', $self)
+    if !$self->{_account_file};
+  return $self->{_account_file};
+}
+
 1;
 
 __END__
